/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master;

import org.log4j.*;

public class Protocol48Handler implements ProtocolVersionHandler
{
    private Category CAT=Category.getInstance(Protocol48Handler.class.getName());
    private Category LOG_CAT=Category.getInstance("q3master.protocol");

    protected final static byte[] RSP_EOT={92,69,79,84};

    public byte[][] createResponse(ServerData[] servers)
    {
        byte[][] responses=new byte[servers.length][];

        for (int serverNum=0;serverNum<servers.length;serverNum++)
        {
            responses[serverNum]=createResponse(servers[serverNum]);
        }
        return responses;
    }

    protected byte[] createResponse(ServerData server)
    {
        int responseLength=4+ //4 times 0xff
                           RSP_STR_GETSERVERS.length()+ // length of response start string
                           7+ // 2 bytes port, 4 bytes address, 1 byte '\\'
                           4; // 1 entry for "EOT"
        byte[] response=new byte[responseLength];
        response[0]=-1;
        response[1]=-1;
        response[2]=-1;
        response[3]=-1;
        for (int i=0;i<RSP_STR_GETSERVERS.length();i++)
        {
            response[i+4]=(byte)RSP_STR_GETSERVERS.charAt(i);
        }
        int responseOffset=4+RSP_STR_GETSERVERS.length();

        response[responseOffset]=92; // '\\' as end-of-record marker

        response[responseOffset+1]=server._serverAddress.getAddress()[0];
        response[responseOffset+2]=server._serverAddress.getAddress()[1];
        response[responseOffset+3]=server._serverAddress.getAddress()[2];
        response[responseOffset+4]=server._serverAddress.getAddress()[3];
        response[responseOffset+5]=(byte)(server._serverPort>>8); // high byte of port
        response[responseOffset+6]=(byte)(server._serverPort&0xff); // low byte of port
        responseOffset+=7;

        for (int i=0;i<RSP_EOT.length;i++)
        {
            response[responseOffset++]=RSP_EOT[i];
        }
        return response;
    }

    public int getProtocolVersion()
    {
        return 48;
    }
}